      PROGRAM PARTICLESTATISTIC
c
c CALCULATION OF WALL-NORMAL PARTICLE CONCENTRATION PROFILES
c (USING EITHER CHEBYCHEV PLOYNOMIALS OR HYPERBOLIC TANGENT)
c
c C. MARCHIOLI, 02/01/2007
c
c Define precision of variables and include some parameters
c
      INCLUDE 'param.h'
c
c Include more parameters (mainly for use of hyperbolic tangent)
c
      PARAMETER (H = 300)
      PARAMETER (RETAU = 150)
      PARAMETER (NZHT = 96)
      PARAMETER (DELTA = 1.7)
c
c GRID_Z     : array for wall-normal coordinate
c COEFF, DIFF: arrays for hyperoblic tangent binning
c CONC       : array for particle concentration
c
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: GRID_Z
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: DZ
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: COEFF
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: DIFF
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: CONC
c
c P          : array for particle position
c
      DIMENSION P(3,NMAX)
c
c ROP        : particle density
c ROF        : fluid density
c ROPP       : particle-to-fluid density ratio
c DPP        : non-dimensional particle diameter
c
      DOUBLE PRECISION ROP,ROF,ROPP,DPP
      DOUBLE PRECISION GZMIN,GZMAX
      DOUBLE PRECISION SS
      DOUBLE PRECISION ZP
c
c TAUP: particle Stokes number
c
      INTEGER TAUP
      INTEGER NPAR
      INTEGER TIME,ITIME,ITIMEOLD,ITIMEMAX
c
      INTEGER hi
      DOUBLE PRECISION hq
      DOUBLE PRECISION P
c
      CHARACTER (LEN=120) POSPAR
      CHARACTER (LEN=6)   TEMP
      CHARACTER (LEN=6)   STRNTIM
c
c Input parameters (from input file fileconc.inp)
c
      OPEN(UNIT=1,FILE='fileconc.inp',STATUS='OLD')
c
      READ(1,104) POSPAR
      READ(1,*)   NPAR
      READ(1,*)   ITIMEOLD
      READ(1,*)   ITIMEMAX
      READ(1,*)   TAUP
      READ(1,*)   IDUMP
      READ(1,*)   ROP
      READ(1,*)   ROF
      READ(1,*)   ICONCDUMP
      READ(1,*)   NBIN
c
      print *,'Number of Particles:',NPAR
c
      CLOSE(1)
c
c Define the specific (particle-to-fluid) density, ROPP,
c and the non-dimensional value of particle diameter, DPP.
c
       ROPP=ROP/ROF
       DPP = (TAUP*18/ROPP)**0.5
       IF (ROPP.EQ.0) THEN
         print *,'ROPP=',ROPP
         stop
       ENDIF
       IF (DPP.EQ.0) THEN
         print *,'DPP=',ROPP
         stop
       ENDIF
c
c Define min/max value for the wall-normal coordinate
c of the particle
c
        GZMIN = DPP/2.
        GZMAX = H - DPP/2.
        print *,'Particle diameter =', DPP
        print *,'Min wall-normal coord.=', GZMIN
        print *,'Max wall-normal coord.=', GZMAX
c
c Choose between Chebychev polynomials and hyperbolic tangent
c
c-----*
      IF (NBIN.EQ.1) THEN
c-----*
c
c Use Chebiscev polinomials for slabs
c
        ALLOCATE(GRID_Z(NZ))
        ALLOCATE(CONC(NZ))
        NSLITS=NZ
        PI=4.0*ATAN(1.0)
c
        DO K=1,NZ
          SS=(K-1)/DFLOAT(NZ-1)
          GRID_Z(k) = (COS(PI*SS)+1.)*(GZMAX-GZMIN)/2. + GZMIN
c          PRINT *,'GRID_Z(',K,')=',GRID_Z(K)
        ENDDO
c-----*
      ELSE
c-----*
c
c Use hyperbolic tangent for slabs
c
        ALLOCATE(GRID_Z(2*NZHT+1))
        ALLOCATE(CONC(2*NZHT+1))
        ALLOCATE(COEFF(2*NZHT+1))
        ALLOCATE(DIFF(2*NZHT+1))
        NSLITS=2*NZHT+1
c
c        print *,'H=',H
c        print *,'RETAU=',RETAU
c        print *,'DELTA=',DELTA
c        print *,'NZHT=',NZHT
c        print *,'NSLITS=',NSLITS
c
        GRID_Z(NZHT+1)=RETAU
c
c        PRINT *,'GRID_Z(',NZHT+1,')=',GRID_Z(NZHT+1)
c
        DO K=1,NZHT
          COEFF(K)=RETAU*(TANH(DELTA*(K)/DFLOAT(NZHT))/(TANH(DELTA)))
c          print *,'COEFF(',K,')=',COEFF(K)
          DIFF(NZHT+1-K)=COEFF(K)-COEFF(K-1)
          IF (K.EQ.NZHT) THEN
            GRID_Z(1)=0.
          ELSE
            GRID_Z(NZHT+1-K)=GRID_Z(NZHT+2-K)-DIFF(NZHT+1-K)
          ENDIF
c
c          PRINT *,K,DIFF(NZHT+1-k),GRID_Z(NZHT+1-k)
c
        ENDDO
c
      DO K=1,NZHT
        GRID_Z(NZHT+1+K)=H-GRID_Z(NZHT+1-K)
        DIFF(NZHT+1+K)=GRID_Z(NZHT+1+K)-GRID_Z(NZHT+K)
      ENDDO
c
c Write coordinate values into file
c
      OPEN(UNIT=1,FILE='GZ-TANH.DAT',STATUS='UNKNOWN')
      DO K=1,2*NZHT+1
        WRITE (1,300) K,GRID_Z(K)
      ENDDO
      CLOSE(1)
c
c Read the same file to store properly the coordinate
c values into array GRID_Z for later use (mainly for
c proper calculation of the concentration and for proper
c output file format).
c
      OPEN(UNIT=1,FILE='GZ-TANH.DAT',STATUS='UNKNOWN')
      DO K=1,2*NZHT+1
        READ (1,300) KK,GRID_Z(2*NZHT+2-K)
      ENDDO
      CLOSE(1)
c
 300  FORMAT(1x,i3,e15.8)
c
c-----*
      ENDIF
c-----*
c
c =====================
c Time loop begins here
c =====================
c
c Write concentration data files starting from
c time step ITIMEOLD to time step ITIMEMAX with
c jumps equal to IDUMP.
c
      DO 777 ITIME=ITIMEOLD,ITIMEMAX,ICONCDUMP
      print *,'Current time step:',ITIME
c
      OPEN(99,FILE='ISTANTE',FORM='FORMATTED')
      WRITE(99,100) ITIME
      CLOSE(99)
      OPEN(99,FILE='ISTANTE',FORM='FORMATTED')
      READ(99,101) TEMP 
      CLOSE(99,STATUS='DELETE')
c
c Open data file containing particle position
c
      OPEN(UNIT=1,
     &     ACTION='READ',
     &     FILE=TRIM(pospar)//TEMP,
     &     FORM='UNFORMATTED',STATUS='OLD',POSITION='REWIND')
c      
c =========================
c Reading particle position
c =========================
c
         READ  (1) hi,hi,hi,hi,hi,hi
         READ  (1) hq,hq,hq,hq,hq,hq,hq
         READ  (1) (P(1,n),n=1,NPAR)
         READ  (1) (P(2,n),n=1,NPAR)
         READ  (1) (P(3,n),n=1,NPAR)
c
c Just a check
c
c         P3MAX=0.0
c         P3MIN=300.0
c         do n = 1, NPAR
c          P3MAX=MAX(P3MAX,P(3,n))
c          P3MIN=MIN(P3MIN,P(3,n))
c         enddo
c         print *,'Before conversion: P3MAX=',P3MAX
c         print *,'Before conversion: P3MIN=',P3MIN
c
c Convert the wall-normal coordinate of the particle
c into actual wall units. Conversion is needed since
c P(3,n) is computed as sum of two contributions:
c
c 1) the index k associated to the bottom face of the
c    host cell (i.e the cell containing the particle)
c
c plus
c
c 2) the ratio R of the wall-normal particle position
c    within the host cell to the host cell height.
c
c  k=193 ==================== (wall)
c          |    |    |    |   cell #192
c  k=192 --|----|----|----|--
c          |    |    |    |   cell #191
c  k=191 --|----|----|----|--
c          |    |    |    |
c         ...  ...  ...  ...
c          |    |    |    |
c  k=3   --|----|----|----|--
c          |    |    | *  |   <- P(3,n) for particle P will be
c          |    |    | P  |      equal to k=2+R=2/3=2.6666
c  k=2   --|----|----|----|--
c          |    |    |    |   cell #  1
c  k=1   --|----|----|----|--
c          |    |    |    |   cell #  0
c  k=0   ==================== (wall)
c
c using file ../concentrations_marchioli/DATA/GRID.dat
c
      ALLOCATE(DZ(NZHT*2+1))
c
      OPEN(UNIT=33,
     &     ACTION='READ',
     &     FILE='./GRID.dat',
     &     FORM='FORMATTED',STATUS='OLD')
c
         do i = 1, NZHT*2+1
          read (33,*) pp,pp,dz(i)
         enddo
c
      CLOSE(33)
c
         do n = 1, NPAR
          P(3,n)=dz(1+int(P(3,n)))*H+
     &           (P(3,n)-int(P(3,n)))*(dz(int(P(3,n))+1)-dz(int(P(3,n))))*H
         enddo
c
c Just a check
c
c         P3MAX=0.0
c         P3MIN=300.0
c         do n = 1, NPAR
c          P3MAX=MAX(P3MAX,P(3,n))
c          P3MIN=MIN(P3MIN,P(3,n))
c         enddo
c         print *,'After conversion: P3MAX=',P3MAX
c         print *,'After conversion: P3MIN=',P3MIN
c
c Initialize the array for particle concentration
c
        DO K=1,NSLITS
          CONC(k) = 0.
        ENDDO
c
c Initialize the arrays for the maximum of particle
c concentration, CONCMAX, and for the integral value
c of particle concentration in the viscous sublayer,
c CONCTOT.
c
        CONCMAX = 0.
        CONCTOT = 0.
c
c Search for particle location within the slabs and then counts
c the number of particles in each slab.
c
c-------*
        DO NP=1,NPAR
          ZP = P(3,NP)
          IF(ZP.GT.RETAU) ZP=H-ZP
          KTEST = 0
 73       CONTINUE
          KTEST = KTEST + 1
          IF(((ZP-GRID_Z(KTEST))*(ZP-GRID_Z(KTEST+1))).GT.0.) GOTO 73
          CONC(KTEST) = CONC(KTEST)+1.
        ENDDO
c-------*
c
c At this point, the array CONC contains the total number of
c particles in each slab. In the following do loop, CONC is
c normalized to obtain the concentration as C/Co where C and
c Co are particle number densities defined as follows:
c C=CONC(K)/Vslab with: * CONC(K) = number of particles in the
c                                   K-th fluid slab
c                       * Vslab   = volume of the K-th fluid slab
c
c Co=NPAR/Vtot    with: * NPAR    = total number of particles
c                                   released in the channel
c                       * Vtot    = total volume of the flow domain
c
        DO K=(NSLITS-1)/2,NSLITS-1
          CONC(K) = CONC(K) * ABS(GRID_Z(NSLITS)-GRID_Z(1)) /
     &              (DFLOAT(NPAR) * ABS(GRID_Z(K)-GRID_Z(K+1)))
          CONC(K) = CONC(K)/2.
        ENDDO
c
c Search for the maximum value of particle concentration and its location
c
        CONCMAX = CONC(1)
        IK_PART = 1
        DO K=(NSLITS-1)/2,NSLITS-1
         IF(CONCMAX.LT.CONC(K))THEN
           CONCMAX = CONC(K)
           IK_PART = K
         ENDIF
        ENDDO
c
c Compute the integral value of the viscous sublayer particle concentration
c
       DO K=1,NSLITS
         IF(GRID_Z(K).LT.5.OR.GRID_Z(K).GT.295) THEN
           CONCTOT = CONCTOT + CONC(K)
         ENDIF
       ENDDO
       CONCTOT = CONCTOT/2.
c
c      print *,'ITIMEMAX-ITIMEOLD+1=',ITIMEMAX-ITIMEOLD+1
c
      IF((MOD((ITIME-ITIMEOLD),ICONCDUMP).EQ.0).OR.(ITIME.EQ.ITIMEMAX))THEN
        OPEN(UNIT=1002,FILE='TEMP.TXT',STATUS='UNKNOWN')
        WRITE(1002,100) ITIME
        CLOSE(1002,STATUS='KEEP')
        OPEN(UNIT=1002,FILE='TEMP.TXT',STATUS='OLD')
        READ(1002,101) STRNTIM
        CLOSE(1002,STATUS='DELETE')
        DO I=1,6
          IF(strntim(I:I).EQ.' ') strntim(I:I)='0'
        ENDDO
      ENDIF
c
c Open output files
c
      OPEN(UNIT=111,FILE='OUTPUT/concmax',ACCESS='APPEND')
      WRITE(111,200)ITIME,(ITIME-200000)*0.045,CONCMAX,GRID_Z(IK_PART)
c
      OPEN(UNIT=112,FILE='OUTPUT/conctot',ACCESS='APPEND')
      WRITE(112,202)ITIME,(ITIME-200000)*0.045,CONCTOT
c
      IF((MOD((ITIME-ITIMEOLD),ICONCDUMP).EQ.0).OR.(ITIME.EQ.ITIMEMAX))THEN
        OPEN(UNIT=113,FILE='OUTPUT/conc'//STRNTIM,ACCESS='APPEND')
        WRITE(113,*) '#'
        WRITE(113,*) '# Particle Concentration along the Wall-Normal Coordinate'
        WRITE(113,*) '# (profile computed at time',INT((ITIME-200000)*0.045),' in wall units)'
        WRITE(113,*) '#'
        WRITE(113,*) '# Column 1: Grid Node (K)'
        WRITE(113,*) '# Column 2: Non-Dimensional Wall-Normal Coordinate (Z+)'
        WRITE(113,*) '# Column 3: Normalized Particle Concentration (C/Co)'
        WRITE(113,*) '#'
        WRITE(113,*) '# Use columns 2 and 3 for plots'
        WRITE(113,*) '#'
        DO K=(NSLITS+1)/2,NSLITS-1
          WRITE(113,201)K,GRID_Z(K),CONC(K)
c          WRITE(113,200)K,GRID_Z(K),(GRID_Z(K)+GRID_Z(K+1))/2.,CONC(K)
        ENDDO
      ENDIF
c
c Close output files
c
      CLOSE(111)
      CLOSE(112)
      IF((MOD((ITIME-ITIMEOLD),ICONCDUMP).EQ.0).OR.(ITIME.EQ.ITIMEMAX))THEN
        CLOSE(113)
      ENDIF
c
c ===================
c Time loop ends here
c ===================
c
 777  CONTINUE
c
c Formats
c
 100  FORMAT(I6)
 101  FORMAT(A6)
 102  FORMAT(I2.2)
 103  FORMAT(A2)
 104  FORMAT(A120)
 200  FORMAT(I6,3(E16.8))
 201  FORMAT(I3,2(E16.8))
 202  FORMAT(I6,2(E16.8))
c 
      STOP 
      END
